@@ -901,8 +901,16 @@ account_widget_build_irc (EmpathyAccountWidget *self,
   const char *filename)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  empathy_account_widget_irc_build (self, filename,
-    &priv->table_common_settings);
+
+  if (priv->simple)
+    {
+      empathy_account_widget_irc_build_simple (self, filename);
+    }
+  else
+    {
+      empathy_account_widget_irc_build (self, filename,
+        &priv->table_common_settings);
+    }
 }
 
 static void
@@ -1454,6 +1462,23 @@ do_get_property (GObject *object,
     }
 }
 
+static void
+set_apply_button (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  GtkWidget *image;
+
+  /* We can't use the stock button as its accelerator ('A') clashes with the
+   * Add button. */
+  gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), FALSE);
+
+  gtk_button_set_label (GTK_BUTTON (priv->apply_button), _("A_pply"));
+  gtk_button_set_use_underline (GTK_BUTTON (priv->apply_button), TRUE);
+
+  image = gtk_image_new_from_stock (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_image (GTK_BUTTON (priv->apply_button), image);
+}
+
 static void
 presence_changed_cb (TpAccountManager *manager,
     TpConnectionPresenceType state,
@@ -1487,9 +1512,7 @@ presence_changed_cb (TpAccountManager *manager,
     {
       /* We are offline or modifying an existing account, display
        * a Save button */
-      gtk_button_set_image (GTK_BUTTON (priv->apply_button), NULL);
-      gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), TRUE);
-      gtk_button_set_label (GTK_BUTTON (priv->apply_button), GTK_STOCK_APPLY);
+      set_apply_button (self);
     }
 }
 
@@ -1556,7 +1579,7 @@ add_enable_checkbox (EmpathyAccountWidget *self,
   gtk_widget_show (w);
 #else
   priv->enabled_checkbox =
-      gtk_check_button_new_with_label (_("Enabled"));
+      gtk_check_button_new_with_mnemonic (_("_Enabled"));
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->enabled_checkbox),
       is_enabled);
@@ -1712,10 +1735,19 @@ do_constructed (GObject *obj)
   if (!priv->simple)
     {
       GtkWidget *hbox = gtk_hbox_new (TRUE, 3);
+      GtkWidget *image;
 
-      priv->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+      /*  We can't use the stock button as its accelerator ('C') clashes with
+       *  the Close button. */
+      priv->cancel_button = gtk_button_new ();
+      gtk_button_set_label (GTK_BUTTON (priv->cancel_button), _("Ca_ncel"));
+      gtk_button_set_use_underline (GTK_BUTTON (priv->cancel_button), TRUE);
 
-      priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
+      image = gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON);
+      gtk_button_set_image (GTK_BUTTON (priv->cancel_button), image);
+
+      priv->apply_button = gtk_button_new ();
+      set_apply_button (self);
 
       /* We'll change this button to a "Log in" one if we are creating a new
        * account and are connected. */
