@@ -129,9 +129,9 @@ empathy_builder_get_file (const gchar *filename,
 }
 
 void
-empathy_builder_connect (GtkBuilder *gui,
-			 gpointer    user_data,
-			 gchar      *first_object,
+empathy_builder_connect (GtkBuilder  *gui,
+			 gpointer     user_data,
+			 const gchar *first_object,
 			 ...)
 {
 	va_list      args;
@@ -206,6 +206,7 @@ empathy_icon_name_for_presence (TpConnectionPresenceType presence)
 	case TP_CONNECTION_PRESENCE_TYPE_UNKNOWN:
 		return EMPATHY_IMAGE_PENDING;
 	case TP_CONNECTION_PRESENCE_TYPE_UNSET:
+	default:
 		return NULL;
 	}
 
@@ -563,7 +564,7 @@ avatar_file_load_contents_cb (GObject      *object,
 {
 	GFile *file = G_FILE (object);
 	PixbufAvatarFromIndividualClosure *closure = user_data;
-	char *data;
+	char *data = NULL;
 	gsize data_size;
 	struct SizeData size_data;
 	GError *error = NULL;
@@ -616,9 +617,10 @@ out:
 void
 empathy_pixbuf_avatar_from_individual_scaled_async (
 		FolksIndividual     *individual,
-		GAsyncReadyCallback  callback,
 		gint                 width,
 		gint                 height,
+		GCancellable        *cancellable,
+		GAsyncReadyCallback  callback,
 		gpointer             user_data)
 {
 	GFile *avatar_file;
@@ -638,7 +640,7 @@ empathy_pixbuf_avatar_from_individual_scaled_async (
 	if (closure == NULL)
 		goto out;
 
-	g_file_load_contents_async (avatar_file, NULL,
+	g_file_load_contents_async (avatar_file, cancellable,
 			avatar_file_load_contents_cb, closure);
 
 	g_object_unref (result);
