@@ -212,6 +212,41 @@ contact_dispose (GObject *object)
   G_OBJECT_CLASS (empathy_contact_parent_class)->dispose (object);
 }
 
+static void
+contact_constructed (GObject *object)
+{
+  EmpathyContact *contact = (EmpathyContact *) object;
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+  GHashTable *location;
+  TpHandle self_handle;
+  TpHandle handle;
+
+  if (priv->tp_contact == NULL)
+    return;
+
+  priv->presence = empathy_contact_get_presence (contact);
+
+  location = tp_contact_get_location (priv->tp_contact);
+  if (location != NULL)
+    empathy_contact_set_location (contact, location);
+
+  set_capabilities_from_tp_caps (contact,
+      tp_contact_get_capabilities (priv->tp_contact));
+
+  contact_set_avatar_from_tp_contact (contact);
+
+  /* Set is-user property. Note that it could still be the handle is
+   * different from the connection's self handle, in the case the handle
+   * comes from a group interface. */
+  self_handle = tp_connection_get_self_handle (
+      tp_contact_get_connection (priv->tp_contact));
+  handle = tp_contact_get_handle (priv->tp_contact);
+  empathy_contact_set_is_user (contact, self_handle == handle);
+
+  g_signal_connect (priv->tp_contact, "notify",
+    G_CALLBACK (tp_contact_notify_cb), contact);
+}
+
 static void
 empathy_contact_class_init (EmpathyContactClass *class)
 {
@@ -223,6 +258,7 @@ empathy_contact_class_init (EmpathyContactClass *class)
   object_class->dispose = contact_dispose;
   object_class->get_property = contact_get_property;
   object_class->set_property = contact_set_property;
+  object_class->constructed = contact_constructed;
 
   g_object_class_install_property (object_class,
       PROP_TP_CONTACT,
@@ -370,40 +406,104 @@ contact_finalize (GObject *object)
 }
 
 static void
-set_tp_contact (EmpathyContact *contact,
-                TpContact *tp_contact)
+empathy_contact_set_capabilities (EmpathyContact *contact,
+                                  EmpathyCapabilities capabilities)
 {
-  EmpathyContactPriv *priv = GET_PRIV (contact);
-  GHashTable *location;
-  TpHandle self_handle;
-  TpHandle handle;
+  EmpathyContactPriv *priv;
 
-  if (tp_contact == NULL)
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+  priv = GET_PRIV (contact);
+
+  if (priv->capabilities == capabilities)
     return;
 
-  g_assert (priv->tp_contact == NULL);
-  priv->tp_contact = g_object_ref (tp_contact);
-  priv->presence = empathy_contact_get_presence (contact);
+  priv->capabilities = capabilities;
 
-  location = tp_contact_get_location (tp_contact);
-  if (location != NULL)
-    empathy_contact_set_location (contact, location);
+  g_object_notify (G_OBJECT (contact), "capabilities");
+}
 
-  set_capabilities_from_tp_caps (contact,
-      tp_contact_get_capabilities (tp_contact));
+static void
+empathy_contact_set_id (EmpathyContact *contact,
+                        const gchar *id)
+{
+  EmpathyContactPriv *priv;
 
-  contact_set_avatar_from_tp_contact (contact);
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+  g_return_if_fail (id != NULL);
 
-  /* Set is-user property. Note that it could still be the handle is
-   * different from the connection's self handle, in the case the handle
-   * comes from a group interface. */
-  self_handle = tp_connection_get_self_handle (
-      tp_contact_get_connection (tp_contact));
-  handle = tp_contact_get_handle (tp_contact);
-  empathy_contact_set_is_user (contact, self_handle == handle);
+  priv = GET_PRIV (contact);
 
-  g_signal_connect (priv->tp_contact, "notify",
-    G_CALLBACK (tp_contact_notify_cb), contact);
+  /* We temporally ref the contact because it could be destroyed
+   * during the signal emition */
+  g_object_ref (contact);
+  if (tp_strdiff (id, priv->id))
+    {
+      g_free (priv->id);
+      priv->id = g_strdup (id);
+
+      g_object_notify (G_OBJECT (contact), "id");
+      if (EMP_STR_EMPTY (priv->alias))
+          g_object_notify (G_OBJECT (contact), "alias");
+    }
+
+  g_object_unref (contact);
+}
+
+static void
+empathy_contact_set_presence (EmpathyContact *contact,
+                              TpConnectionPresenceType presence)
+{
+  EmpathyContactPriv *priv;
+  TpConnectionPresenceType old_presence;
+
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+  priv = GET_PRIV (contact);
+
+  if (presence == priv->presence)
+    return;
+
+  old_presence = priv->presence;
+  priv->presence = presence;
+
+  g_signal_emit (contact, signals[PRESENCE_CHANGED], 0, presence, old_presence);
+
+  g_object_notify (G_OBJECT (contact), "presence");
+}
+
+static void
+empathy_contact_set_presence_message (EmpathyContact *contact,
+                                      const gchar *message)
+{
+  EmpathyContactPriv *priv = GET_PRIV (contact);
+
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+  if (priv->persona != NULL)
+    {
+      folks_presence_set_presence_message (FOLKS_PRESENCE (priv->persona),
+          message);
+    }
+}
+
+static void
+empathy_contact_set_handle (EmpathyContact *contact,
+                            guint handle)
+{
+  EmpathyContactPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
+  priv = GET_PRIV (contact);
+
+  g_object_ref (contact);
+  if (handle != priv->handle)
+    {
+      priv->handle = handle;
+      g_object_notify (G_OBJECT (contact), "handle");
+    }
+  g_object_unref (contact);
 }
 
 static void
@@ -467,7 +567,7 @@ contact_set_property (GObject *object,
   switch (param_id)
     {
       case PROP_TP_CONTACT:
-        set_tp_contact (contact, g_value_get_object (value));
+        priv->tp_contact = g_value_dup_object (value);
         break;
       case PROP_ACCOUNT:
         g_assert (priv->account == NULL);
@@ -565,33 +665,6 @@ empathy_contact_get_id (EmpathyContact *contact)
   return priv->id;
 }
 
-void
-empathy_contact_set_id (EmpathyContact *contact,
-                        const gchar *id)
-{
-  EmpathyContactPriv *priv;
-
-  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-  g_return_if_fail (id != NULL);
-
-  priv = GET_PRIV (contact);
-
-  /* We temporally ref the contact because it could be destroyed
-   * during the signal emition */
-  g_object_ref (contact);
-  if (tp_strdiff (id, priv->id))
-    {
-      g_free (priv->id);
-      priv->id = g_strdup (id);
-
-      g_object_notify (G_OBJECT (contact), "id");
-      if (EMP_STR_EMPTY (priv->alias))
-          g_object_notify (G_OBJECT (contact), "alias");
-    }
-
-  g_object_unref (contact);
-}
-
 const gchar *
 empathy_contact_get_alias (EmpathyContact *contact)
 {
@@ -880,28 +953,6 @@ empathy_contact_get_presence (EmpathyContact *contact)
   return priv->presence;
 }
 
-void
-empathy_contact_set_presence (EmpathyContact *contact,
-                              TpConnectionPresenceType presence)
-{
-  EmpathyContactPriv *priv;
-  TpConnectionPresenceType old_presence;
-
-  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-
-  priv = GET_PRIV (contact);
-
-  if (presence == priv->presence)
-    return;
-
-  old_presence = priv->presence;
-  priv->presence = presence;
-
-  g_signal_emit (contact, signals[PRESENCE_CHANGED], 0, presence, old_presence);
-
-  g_object_notify (G_OBJECT (contact), "presence");
-}
-
 const gchar *
 empathy_contact_get_presence_message (EmpathyContact *contact)
 {
@@ -917,21 +968,6 @@ empathy_contact_get_presence_message (EmpathyContact *contact)
   return NULL;
 }
 
-void
-empathy_contact_set_presence_message (EmpathyContact *contact,
-                                      const gchar *message)
-{
-  EmpathyContactPriv *priv = GET_PRIV (contact);
-
-  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-
-  if (priv->persona != NULL)
-    {
-      folks_presence_set_presence_message (FOLKS_PRESENCE (priv->persona),
-          message);
-    }
-}
-
 guint
 empathy_contact_get_handle (EmpathyContact *contact)
 {
@@ -947,25 +983,6 @@ empathy_contact_get_handle (EmpathyContact *contact)
   return priv->handle;
 }
 
-void
-empathy_contact_set_handle (EmpathyContact *contact,
-                            guint handle)
-{
-  EmpathyContactPriv *priv;
-
-  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-
-  priv = GET_PRIV (contact);
-
-  g_object_ref (contact);
-  if (handle != priv->handle)
-    {
-      priv->handle = handle;
-      g_object_notify (G_OBJECT (contact), "handle");
-    }
-  g_object_unref (contact);
-}
-
 EmpathyCapabilities
 empathy_contact_get_capabilities (EmpathyContact *contact)
 {
@@ -978,24 +995,6 @@ empathy_contact_get_capabilities (EmpathyContact *contact)
   return priv->capabilities;
 }
 
-void
-empathy_contact_set_capabilities (EmpathyContact *contact,
-                                  EmpathyCapabilities capabilities)
-{
-  EmpathyContactPriv *priv;
-
-  g_return_if_fail (EMPATHY_IS_CONTACT (contact));
-
-  priv = GET_PRIV (contact);
-
-  if (priv->capabilities == capabilities)
-    return;
-
-  priv->capabilities = capabilities;
-
-  g_object_notify (G_OBJECT (contact), "capabilities");
-}
-
 gboolean
 empathy_contact_is_user (EmpathyContact *contact)
 {
