@@ -41,12 +41,12 @@
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
 #include <libempathy/empathy-chatroom-manager.h>
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-list.h>
 
 #include <libempathy-gtk/empathy-images.h>
-#include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-contact-dialogs.h>
 #include <libempathy-gtk/empathy-log-window.h>
 #include <libempathy-gtk/empathy-geometry.h>
@@ -94,6 +94,7 @@ typedef struct {
 	GtkUIManager *ui_manager;
 	GtkAction   *menu_conv_insert_smiley;
 	GtkAction   *menu_conv_favorite;
+	GtkAction   *menu_conv_always_urgent;
 	GtkAction   *menu_conv_toggle_contacts;
 
 	GtkAction   *menu_edit_cut;
@@ -518,9 +519,10 @@ chat_window_icon_update (EmpathyChatWindowPriv *priv)
 		gtk_window_set_icon_name (GTK_WINDOW (priv->dialog),
 					  EMPATHY_IMAGE_MESSAGE);
 	} else {
-		empathy_conf_get_bool (empathy_conf_get (),
-				       EMPATHY_PREFS_CHAT_AVATAR_IN_ICON,
-				       &avatar_in_icon);
+		GSettings *gsettings = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+
+		avatar_in_icon = g_settings_get_boolean (gsettings,
+				EMPATHY_PREFS_CHAT_AVATAR_IN_ICON);
 
 		if (n_chats == 1 && avatar_in_icon) {
 			remote_contact = empathy_chat_get_remote_contact (priv->current_chat);
@@ -533,6 +535,8 @@ chat_window_icon_update (EmpathyChatWindowPriv *priv)
 		} else {
 			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 		}
+
+		g_object_unref (gsettings);
 	}
 }
 
@@ -785,8 +789,16 @@ chat_window_conv_activate_cb (GtkAction         *action,
 		DEBUG ("This room %s favorite", found ? "is" : "is not");
 		gtk_toggle_action_set_active (
 			GTK_TOGGLE_ACTION (priv->menu_conv_favorite), found);
+
+		if (chatroom != NULL)
+			found = empathy_chatroom_is_always_urgent (chatroom);
+
+		gtk_toggle_action_set_active (
+			GTK_TOGGLE_ACTION (priv->menu_conv_always_urgent),
+			found);
 	}
 	gtk_action_set_visible (priv->menu_conv_favorite, is_room);
+	gtk_action_set_visible (priv->menu_conv_always_urgent, is_room);
 
 	/* Show contacts menu */
 	g_object_get (priv->current_chat,
@@ -828,19 +840,38 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 	account = empathy_chat_get_account (priv->current_chat);
 	room = empathy_chat_get_id (priv->current_chat);
 
-	chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
-						  account, room);
-
-	if (chatroom == NULL) {
-		const gchar *name;
-
-		name = empathy_chat_get_name (priv->current_chat);
-		chatroom = empathy_chatroom_new_full (account, room, name, FALSE);
-		empathy_chatroom_manager_add (priv->chatroom_manager, chatroom);
-		g_object_unref (chatroom);
-	}
+	chatroom = empathy_chatroom_manager_ensure_chatroom (
+		     priv->chatroom_manager,
+		     account,
+		     room,
+		     empathy_chat_get_name (priv->current_chat));
 
 	empathy_chatroom_set_favorite (chatroom, active);
+	g_object_unref (chatroom);
+}
+
+static void
+chat_window_always_urgent_toggled_cb (GtkToggleAction   *toggle_action,
+				 EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gboolean               active;
+	TpAccount             *account;
+	const gchar           *room;
+	EmpathyChatroom       *chatroom;
+
+	active = gtk_toggle_action_get_active (toggle_action);
+	account = empathy_chat_get_account (priv->current_chat);
+	room = empathy_chat_get_id (priv->current_chat);
+
+	chatroom = empathy_chatroom_manager_ensure_chatroom (
+		     priv->chatroom_manager,
+		     account,
+		     room,
+		     empathy_chat_get_name (priv->current_chat));
+
+	empathy_chatroom_set_always_urgent (chatroom, active);
+	g_object_unref (chatroom);
 }
 
 static void
@@ -856,7 +887,7 @@ chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
 }
 
 static void
-got_contact_cb (EmpathyTpContactFactory *factory,
+got_contact_cb (TpConnection            *connection,
                 EmpathyContact          *contact,
                 const GError            *error,
                 gpointer                 user_data,
@@ -900,20 +931,15 @@ chat_window_invite_participant_activate_cb (GtkAction         *action,
 
 	if (response == GTK_RESPONSE_ACCEPT) {
 		TpConnection *connection;
-		EmpathyTpContactFactory *factory;
 		const char *id;
 
 		id = empathy_contact_selector_dialog_get_selected (
-				EMPATHY_CONTACT_SELECTOR_DIALOG (dialog), NULL);
+				EMPATHY_CONTACT_SELECTOR_DIALOG (dialog), NULL, NULL);
 		if (EMP_STR_EMPTY (id)) goto out;
 
 		connection = tp_channel_borrow_connection (channel);
-		factory = empathy_tp_contact_factory_dup_singleton (connection);
-
-		empathy_tp_contact_factory_get_from_id (factory, id,
+		empathy_tp_contact_factory_get_from_id (connection, id,
 			got_contact_cb, tp_chat,  NULL, NULL);
-
-		g_object_unref (factory);
 	}
 
 out:
@@ -1241,15 +1267,21 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		return;
 	} else {
-		empathy_conf_get_bool (empathy_conf_get (),
-				       EMPATHY_PREFS_NOTIFICATIONS_FOCUS, &res);
+		GSettings *gsettings = g_settings_new (
+				EMPATHY_PREFS_NOTIFICATIONS_SCHEMA);
+
+		res = g_settings_get_boolean (gsettings,
+				EMPATHY_PREFS_NOTIFICATIONS_FOCUS);
+
+		g_object_unref (gsettings);
+
 		if (!res) {
 			return;
 		}
 	}
 
 	sender = empathy_message_get_sender (message);
-	header = empathy_contact_get_name (sender);
+	header = empathy_contact_get_alias (sender);
 	body = empathy_message_get_body (message);
 	escaped = g_markup_escape_text (body, -1);
 	has_x_canonical_append = empathy_notify_manager_has_capability (
@@ -1276,8 +1308,8 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 
 		notify_notification_set_timeout (notification, NOTIFY_EXPIRES_DEFAULT);
 
-		g_signal_connect (notification, "closed",
-				  G_CALLBACK (chat_window_notification_closed_cb), window);
+		tp_g_signal_connect_object (notification, "closed",
+				  G_CALLBACK (chat_window_notification_closed_cb), window, 0);
 
 		if (has_x_canonical_append) {
 			notify_notification_set_hint_string (notification,
@@ -1360,11 +1392,27 @@ chat_window_new_message_cb (EmpathyChat       *chat,
 	/* If empathy_chat_is_room () returns TRUE, that means it's a named MUC.
 	 * If empathy_chat_get_remote_contact () returns NULL, that means it's
 	 * an unamed MUC (msn-like).
-	 * In case of a MUC, we set urgency only if the message contains our
-	 * alias. */
+	 * In case of a MUC, we set urgency if either:
+	 *   a) the chatroom's always_urgent property is TRUE
+	 *   b) the message contains our alias
+	 */
 	if (empathy_chat_is_room (chat) ||
 	    empathy_chat_get_remote_contact (chat) == NULL) {
-		needs_urgency = empathy_message_should_highlight (message);
+		TpAccount             *account;
+		const gchar           *room;
+		EmpathyChatroom       *chatroom;
+
+		account = empathy_chat_get_account (chat);
+		room = empathy_chat_get_id (chat);
+
+		chatroom = empathy_chatroom_manager_find (priv->chatroom_manager,
+							  account, room);
+
+		if (empathy_chatroom_is_always_urgent (chatroom)) {
+			needs_urgency = TRUE;
+		} else {
+			needs_urgency = empathy_message_should_highlight (message);
+		}
 	} else {
 		needs_urgency = TRUE;
 	}
@@ -1405,15 +1453,15 @@ chat_window_detach_hook (GtkNotebook *source,
 
 	empathy_chat_window_move_chat (window, new_window, chat);
 
-	gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
 	gtk_widget_show (priv->dialog);
+	gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
 
 	return NULL;
 }
 
 static void
 chat_window_page_switched_cb (GtkNotebook      *notebook,
-			      GtkNotebookPage  *page,
+			      gpointer          ignore, /* see note below */
 			      gint	        page_num,
 			      EmpathyChatWindow *window)
 {
@@ -1425,6 +1473,9 @@ chat_window_page_switched_cb (GtkNotebook      *notebook,
 
 	priv = GET_PRIV (window);
 
+	/* N.B. in GTK+ 3 child is passed as the first argument to the signal,
+	 * but we can't use that while trying to retain GTK+ 2.x compatibility.
+	 */
 	child = gtk_notebook_get_nth_page (notebook, page_num);
 	chat = EMPATHY_CHAT (child);
 
@@ -1687,14 +1738,8 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		}
 
 		if (!chat) {
-			TpConnection *connection;
-
-			connection = tp_account_get_connection (account);
-
-			if (connection) {
-				empathy_dispatcher_chat_with_contact_id (
-					connection, contact_id, gtk_get_current_event_time (), NULL, NULL);
-			}
+			empathy_dispatcher_chat_with_contact_id (
+				account, contact_id, gtk_get_current_event_time ());
 
 			g_strfreev (strv);
 			return;
@@ -1716,7 +1761,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 
 		/* Added to take care of any outstanding chat events */
 		empathy_chat_window_present_chat (chat,
-			EMPATHY_DISPATCHER_NON_USER_ACTION);
+			TP_USER_ACTION_TIME_NOT_USER_ACTION);
 
 		/* We should return TRUE to remove the data when doing
 		 * GDK_ACTION_MOVE, but we don't here otherwise it has
@@ -1865,6 +1910,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 				       "ui_manager", &priv->ui_manager,
 				       "menu_conv_insert_smiley", &priv->menu_conv_insert_smiley,
 				       "menu_conv_favorite", &priv->menu_conv_favorite,
+				       "menu_conv_always_urgent", &priv->menu_conv_always_urgent,
 				       "menu_conv_toggle_contacts", &priv->menu_conv_toggle_contacts,
 				       "menu_edit_cut", &priv->menu_edit_cut,
 				       "menu_edit_copy", &priv->menu_edit_copy,
@@ -1883,6 +1929,7 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
+			      "menu_conv_always_urgent", "toggled", chat_window_always_urgent_toggled_cb,
 			      "menu_conv_toggle_contacts", "toggled", chat_window_contacts_toggled_cb,
 			      "menu_conv_invite_participant", "activate", chat_window_invite_participant_activate_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
@@ -2027,12 +2074,14 @@ empathy_chat_window_new (void)
 EmpathyChatWindow *
 empathy_chat_window_get_default (gboolean room)
 {
+	GSettings *gsettings = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
 	GList    *l;
 	gboolean  separate_windows = TRUE;
 
-	empathy_conf_get_bool (empathy_conf_get (),
-			      EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
-			      &separate_windows);
+	separate_windows = g_settings_get_boolean (gsettings,
+			EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS);
+
+	g_object_unref (gsettings);
 
 	if (separate_windows) {
 		/* Always create a new window */
@@ -2100,12 +2149,14 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 
 	/* If this window has just been created, position it */
 	if (priv->chats == NULL) {
+		GSettings *gsettings = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
 		const gchar *name = "chat-window";
 		gboolean     separate_windows;
 
-		empathy_conf_get_bool (empathy_conf_get (),
-				       EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
-				       &separate_windows);
+		separate_windows = g_settings_get_boolean (gsettings,
+				EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS);
+
+		g_object_unref (gsettings);
 
 		if (separate_windows) {
 			name = empathy_chat_get_id (chat);
@@ -2304,15 +2355,12 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
 	/* Don't force the window to show itself when it wasn't
 	 * an action by the user
 	 */
-	if (timestamp == EMPATHY_DISPATCHER_NON_USER_ACTION)
+	if (!tp_user_action_time_should_present (timestamp, &x_timestamp))
 		return;
 
 	priv = GET_PRIV (window);
 
-	if (timestamp == EMPATHY_DISPATCHER_CURRENT_TIME) {
-		x_timestamp = GDK_CURRENT_TIME;
-	} else {
-		x_timestamp = CLAMP (timestamp, 0, G_MAXUINT32);
+	if (x_timestamp != GDK_CURRENT_TIME) {
 		/* Don't present or switch tab if the action was earlier than the
 		 * last actions X time, accounting for overflow and the first ever
 		* presentation */
